#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-data-single-catalog-oracle-config v. 2.1.2
#
# Post-installation script for configuring the gLite Data Management Single Catalog Oracle Node
# Robert Harakaly  < robert.harakaly@cern.ch >
#
# Version info: $Id: glite-data-single-catalog-oracle-config.py,v 1.28.2.7 2006/01/22 19:30:30 dimeglio Exp $
#
# Usage: python glite-data-single-catalog-oracle-config [-c|-v|-h|-siteconfig=<site config url>
# |--start|--stop|--remove|--status|--configure|--help]
#        -c                             print configuration
#        -v                             print version
#        -h,--help                      print usage info
#        -siteconfig=<site config url>  use site configuration file
#        --start                        start the service
#        --stop                         start the service
#        --remove                       remove the database service
#        --status                       shows the status of the service
#        --configure                    configures the service
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
#####################################################################################

import os,string,socket,time
import sys
import getopt
import commands
import shutil

from  gLiteInstallerLib import gLib
from  gLiteInstallerLib import ConfigParams
from  gridSecurity import gridSecurity
from tomcat5 import Secure_tomcat5
from gliteRgmaServicetool import gliteRgmaServicetoolInstance
from gliteRgmaServicetool import gliteRgmaServicetool

# Set global variables here 
global params                        # all config values from the XML file

class glite_data_single_catalog_oracle:

    def __init__(self):
        self.verbose = 0
        self.version = "2.1.2"
        self.name = "glite-data-single-catalog-oracle"
        self.friendly_name = "Data Management Single Catalog Oracle Service"
        self.glib = params['gLib']
        self.tomcat = Secure_tomcat5(params)
        
    #-------------------------------------------------------------------------------
    # Banner 
    #-------------------------------------------------------------------------------

    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
    #-------------------------------------------------------------------------------
    # Copyright 
    #-------------------------------------------------------------------------------

    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'

    #-------------------------------------------------------------------------------
    # Version
    #-------------------------------------------------------------------------------

    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    
    #-------------------------------------------------------------------------------
    # Usage
    #-------------------------------------------------------------------------------

    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()
    
        print """Usage: \n
Edit the configuration file %s.cfg.xml in
%s/etc.config/templates\n
save it as %s/etc/config/%s.cfg.xml
and run the script as follows\n 
python %s-config [OPTION...]""" % (self.name, os.environ['GLITE_LOCATION'], \
        os.environ['GLITE_LOCATION'], self.name, self.name)

        print '    -c, --checkconf                       print the service configuration'
        print '    -v, --version                         print the version of the configuration script'
        print '    -h, --help                            print this usage information'
        print '    --start                               start the service'
        print '    --stop                                stop the service'
        print '    --remove                              remove the database service'
        print '    --status                              print service status (0=running,1=idle)'
        print '    --configure                           configure the service'
        print '    --siteconfig=<site config url>        use site configuration file'
        print '\n'

    #-------------------------------------------------------------------------------
    # All the configuration code goes here
    #-------------------------------------------------------------------------------

    def configure(self):
        #--------------------------------------------------------------
        # Check Oracle instanclient libs
        #--------------------------------------------------------------
        if params.has_key('catalog-service-fr.oracle-instantclient.location'):
            if params['catalog-service-fr.oracle-instantclient.location'] != "":
                if not os.path.exists(params['catalog-service-fr.oracle-instantclient.location']):
                    glib.printErrorMessage("The Oracle InstantClient cannot be found at the specified location")
                    glib.printErrorMessage(params['catalog-service-fr.oracle-instantclient.location'])
                    glib.printErrorMessage("Configuration cannot continue")
                    return 1

        #--------------------------------------------------------
        # Check java installation
        #--------------------------------------------------------
        if glib.checkJava() != 0:
            return 1


        #-------------------------------------------------------------------------------
        # VO list notice
        #-------------------------------------------------------------------------------
    
        print "\nThis Catalog will be configured to serve the following VOs:"
        for vo in glib.getVoList():
            print "=> %s" % vo
            
        #--------------------------------------------------------
        # Configure the Security Utilities
        #--------------------------------------------------------
        
        if os.system("python %s/glite-security-utils-config.py --subservice" % glib.getScriptPath()):
            print "Configuring gLite security                                ",
            glib.printFailedMessage()
            return 1

        print "Configuring gLite security                                ",
        glib.printOkMessage()
            
        #-------------------------------------------------------------------------------
        # Create vomsdir and check for VOMS certificates
        #-------------------------------------------------------------------------------

        if gsec.createVomsdir():
            return 1

        #--------------------------------------------------------
        # Check configuration values
        #--------------------------------------------------------
        
        print "Checking configuration values                             ",        

        # mysql root password
        if params.has_key('mysql.root.password'):
            mysql_root_password = params['mysql.root.password']
        else:
            mysql_root_password = ""    

        # allow unsecure port
        if params.has_key('allow.unsecure.port'):
            allow_unsecure_port = params['allow.unsecure.port']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"allow.unsecure.port\" not found.")
            return 1

        glib.printOkMessage()
        
        #--------------------------------------------------------
        # Configure Tomcat
        #--------------------------------------------------------
        
        if not params.has_key('CATALINA_HOME'):
            if self.glib.check_existing("/var/lib/tomcat5"):
                glib.printErrorMessage("Cannot find tomcat")
                return 1
            else:
                glib.export("CATALINA_HOME","/var/lib/tomcat5") 

        if self.glib.check_existing("%s/conf/Catalina/localhost" % params['CATALINA_HOME']):
            glib.printErrorMessage("Tomcat webapps config directory does not exist")
            return 1

        if self.tomcat.configure() != 0:
            return 1
                    
        ##################################
        #    Copy Oracle jar file in tomcat location
        ##################################
        os.system("cp %s/lib/ojdbc14.jar %s/common/lib/" % (params['catalog-service-fr.oracle-instantclient.location'], os.environ["CATALINA_HOME"]))
        os.system("cp %s/lib/classes12.jar %s/common/lib/" % (params['catalog-service-fr.oracle-instantclient.location'], os.environ["CATALINA_HOME"]))


        ##################################
        #    Check for Oracle JDBC drivers
        ##################################
        if not os.path.exists('%s/common/lib/classes12.jar' % os.environ["CATALINA_HOME"]) \
            or not os.path.exists('%s/common/lib/ojdbc14.jar' % os.environ["CATALINA_HOME"]):
                glib.printErrorMessage("#####################################################")
                glib.printErrorMessage("Oracle drivers cannot be found. Cannot continue. %s" % os.environ["CATALINA_HOME"])
                glib.printErrorMessage("Please install the Oracle JDBC drivers into the  directory %s/common/lib/" % os.environ["CATALINA_HOME"])
                glib.printErrorMessage("and restart the configuration script")
                glib.printErrorMessage("#####################################################")
                return 1

        ################################
        #    Configure all instances
        ################################
        instances = self.glib.getInstances('FiremanCatalog')
        if len(instances) == 0:
            self.configureInstance()
        else:
            for name in instances.keys():
                self.configureInstance(name)
                
        #-------------------------------------------------------------------
        # RGMA servicetool: configure servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
    
            # Instantiate the rgma-servicetool class
            rgmaServicetool = gliteRgmaServicetool()
            rgmaServicetool.verbose = self.verbose
            
            # Create Fireman Catalog instances
            for vo in glib.getVoList():
                
                serviceId = glib.fq_hostname + "_" + vo + "_" + params['firemanCatalog.serviceType']
                servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
                
                # set params
                servicetoolInstance.setServiceName('Oracle Fireman Catalog service for %s at %s' % (vo,glib.fq_hostname))
                servicetoolInstance.setServiceType(params['firemanCatalog.serviceType'])
                servicetoolInstance.setServiceVersion(self.version)
                servicetoolInstance.setStatusScript('wget -S -r https://%s:8443/%s/glite-data-catalog-service-fr/services/FiremanCatalog' % (glib.fq_hostname,vo))
                servicetoolInstance.setEndpoint('https://%s:8443/%s/glite-data-catalog-service-fr/services/FiremanCatalog' % (glib.fq_hostname,vo))
                servicetoolInstance.setWsdlUrl('https://%s:8443/%s/glite-data-catalog-service-fr/services/FiremanCatalog?wsdl' % (glib.fq_hostname,vo))
                # add instance to the gLite configuration
                if servicetoolInstance.add() == 1:
                    return 1
                
                # Create SEIndex instance
                serviceId = glib.fq_hostname + "_" + vo + "_" + params['seIndex.serviceType']
                servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
                
                # set params
                servicetoolInstance.setServiceName('Oracle SE Index service for %s at %s' % (vo,glib.fq_hostname))
                servicetoolInstance.setServiceType(params['seIndex.serviceType'])
                servicetoolInstance.setServiceVersion(self.version)
                servicetoolInstance.setStatusScript('wget -S -r https://%s:8443/%s/glite-data-catalog-service-fr/services/SEIndex' % (glib.fq_hostname,vo))
                servicetoolInstance.setEndpoint('https://%s:8443/%s/glite-data-catalog-service-fr/services/SEIndex' % (glib.fq_hostname,vo))
                servicetoolInstance.setWsdlUrl('https://%s:8443/%s/glite-data-catalog-service-fr/services/SEIndex?wsdl' % (glib.fq_hostname,vo))
                
                # add instance to the gLite configuration
                if servicetoolInstance.add() == 1:
                    return 1
                
            # Configure servicetool
            if rgmaServicetool.configure(glib):
                # error in configuring servicetool
                return 1
            
        return 0
    
    def configureInstance(self, name='default'):
        if name == 'default':
            name = 'glite-data-catalog-service-fr:default'
            instanceParams = params
        else:
            instanceParams = self.glib.getInstanceParams(name)

        ################################
        #    Check schemas
        ################################
        sqlplusConnectString = "%s/%s@%s:%s/%s @%s" %(instanceParams['catalog-service-fr.DBUSER'],\
            instanceParams['catalog-service-fr.DBPASSWORD'],instanceParams['catalog-service-fr.DBHOST'],\
            instanceParams['catalog-service-fr.DBPORT'],instanceParams['catalog-service-fr.DBSERVICENAME'],\
            os.environ['GLITE_LOCATION'])

        command = "sqlplus %s/etc/glite-data-catalog-service-fr/schema/fm_oracle_get-version.sql" %(sqlplusConnectString)
        file = os.popen(command, 'r')
        version = file.read()
        schemaExists = False
        if file.close() is None:
            print "DB version: %s" %version
            lines = str(version).split('\n')
            while lines:
                oldSchema = string.find(lines[0], "glite-data-catalog-service-fr") 
                del lines[0]
                if oldSchema > 0:
                    glib.printWarningMessage("There is a DB schema already loaded in the database, so the new one will not be loaded.")
                    glib.printWarningMessage("To replace the schema and database, please remove the instance using the --remove command of this script")
                    schemaExists = True
                    break

        if not schemaExists:
            ################################
            #    Load new schema
            ################################
            command = "sqlplus %s/etc/glite-data-catalog-service-fr/schema/fm_oracle_init.sql" %(sqlplusConnectString)
            print "Command: %s" %command
            file = os.popen(command,'w',0)
            file.write('quit')
            file.close()

        ################################
        #    Replace tokens in the context.xml template file
        ################################
        warpath = "%s/webapps/%s/glite-data-catalog-service-fr" \
                % (os.environ["CATALINA_HOME"],instanceParams['vo.name'])
        if os.path.isdir(warpath):
                shutil.rmtree(warpath)
                print "Removed existing web application from ", warpath, "\n"

        inFile = open("%s/etc/glite-data-catalog-service-fr/context.xml.template" % os.environ['GLITE_LOCATION'],"r")
        outFile = open("%s/conf/Catalina/localhost/glite-data-catalog-service-fr.%s.xml" % \
            (os.environ["CATALINA_HOME"], name.split(':')[1]) ,"w")
        for line in inFile.readlines():
            line = line.replace("@PATH@", "/%s/glite-data-catalog-service-fr" % instanceParams['vo.name'])
            line = replaceTags(line, instanceParams, 'catalog-service-fr')
            outFile.write(line)
        outFile.close()
        inFile.close()
        os.system("cp %s/lib/glite-data-common/*.jar /usr/share/tomcat5/common/lib/" \
            % os.environ['GLITE_LOCATION'])
        return 0

    def remove(self):
        ################################
        #    Remove all instances
        ################################

        instances = glib.getInstances('FiremanCatalog')
        if len(instances) > 0:
            for name in instances.keys():
                self.removeInstance(name)
        return 0

    def removeInstance(self, name='default:default'):

        print "Remove glite-data-catalog-service-fr instance: %s" % name
        instanceParams = glib.getInstanceParams(name)

        ################################
        #    Remove context.xml file
        ################################
        context = "%s/conf/Catalina/localhost/glite-data-catalog-service-fr.%s.xml" % (os.environ["CATALINA_HOME"], name.split(':')[1])
        print "Removing context: %s" % context
        if os.path.exists(context):
            os.remove(context)


        ################################
        #    Check schemas
        ################################
        sqlplusConnectString = "%s/%s@%s:%s/%s @%s" %(instanceParams['catalog-service-fr.DBUSER'],\
            instanceParams['catalog-service-fr.DBPASSWORD'],instanceParams['catalog-service-fr.DBHOST'],\
            instanceParams['catalog-service-fr.DBPORT'],instanceParams['catalog-service-fr.DBSERVICENAME'],\
            os.environ['GLITE_LOCATION'])

        command = "sqlplus %s/etc/glite-data-catalog-service-fr/schema/fm_oracle_get-version.sql" %(sqlplusConnectString)
        file = os.popen(command, 'r')
        version = file.read()
        if file.close() is None:
            print "DB version: %s" %version
            lines = str(version).split('\n')
            schemaExists = 0
            while lines:
                lineno = string.find(lines[0], "glite-data-catalog-service-fr")
                del lines[0]
                if lineno >0:
                    schemaExists = 1
            if schemaExists == 0:
                print "No schema found: nothing to do!"
                return 0


        ################################
        #    Remove schemas
        ################################
        print "Removing schema..."
        command = "sqlplus %s/etc/glite-data-catalog-service-fr/schema/fm_oracle_clean.sql" %(sqlplusConnectString)

        print "Command: %s" %command
        file = os.popen(command,'w',0)
        file.write('quit')
        file.close()

    def start(self): 

        if len(os.listdir(params['X509_VOMS_DIR'])) == 0:
            print("\n\n")
            glib.printErrorMessage("No VOMS Server host certificates could be found in %s" % params['X509_VOMS_DIR'])
            glib.printErrorMessage("Before using the %s," % service.friendly_name)
            glib.printErrorMessage("make sure that the host certificate of each used VOMS")
            glib.printErrorMessage("Server is installed in the %s directory." % params['X509_VOMS_DIR'])
            return 1

        self.tomcat.start()
        
        #-------------------------------------------------------------------
        # Start Servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
    
            errorcode = rgmaServicetool.start()
            if (errorcode != 0):
                return 1
            
        return 0
    
    def stop(self):
        
        error_level = 0
        
        if self.tomcat.stop() != 0:
            error_level = 1
        
        #-------------------------------------------------------------------
        # Servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
    
            if rgmaServicetool.stop():
                error_level = 1
    
        return error_level
        
        
    #################################################################
    # status
    #
    # print status of catalog
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def status(self):
        
        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "    Checking status of %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"

        error_level = 0
        
        if params.has_key('host.certificate.file'):
            hostcert_file = params['host.certificate.file']
        else:
            hostcert_file = "/etc/grid-security/hostcert.pem"

        if params.has_key('host.key.file'):    
            hostkey_file  = params['host.key.file']
        else:
            hostkey_file = "/etc/grid-security/hostkey.pem"

        # check for each VO:
        instances = self.glib.getInstances('FiremanCatalog')
        if len(instances) > 0:
            for name in instances.keys():
                
                instanceParams = self.glib.getInstanceParams(name)

                # get VO name
                if instanceParams.has_key('vo.name'):
                    vo_name = instanceParams['vo.name']

                    if self.verbose:
                        print "Checking status of VO '%s': " % vo_name,
                    status = self.glib.checkHTTPSconnection("localhost", "/%s/glite-data-catalog-service-fr/services/FiremanCatalog?method=getVersion" % vo_name, 8443, hostkey_file, hostcert_file)

                    if status[0]:
                        if self.verbose:
                            print "not running. (%s)" % status[2]
                        error_level = 1
                    else:
                        print("running.")

        if error_level:
            print "\n%s is not running." % self.friendly_name
        else:
            print "\n%s is running." % self.friendly_name
            
        #-------------------------------------------------------------------
        # Servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
    
            if rgmaServicetool.status() != 0:
                error_level = 1
    
        return error_level
        
#-------------------------------------------------------------------------------
# Set all environment variables
#-------------------------------------------------------------------------------

def loadDefaults(params):

    params['GLITE_LOCATION'] = "/opt/glite"
    params['mysql.root.password'] = ""
    
    params['firemanCatalog.serviceType'] = 'org.glite.FiremanCatalog'
    params['seIndex.serviceType'] = 'org.glite.SEIndex'
    
#-------------------------------------------------------------------------------
# Set all environment variables
#-------------------------------------------------------------------------------

def set_env():
    
    glib.export('GLITE_LOCATION');
    glib.export('GLITE_LOCATION_VAR');
    if not os.path.exists(os.environ['GLITE_LOCATION_VAR']):
        os.mkdir(os.environ['GLITE_LOCATION_VAR'],0755)
    glib.export('GLITE_LOCATION_LOG');
    if not os.path.exists(os.environ['GLITE_LOCATION_LOG']):
        os.mkdir(os.environ['GLITE_LOCATION_LOG'],0755)
    glib.export('GLITE_LOCATION_TMP');
    if not os.path.exists(os.environ['GLITE_LOCATION_TMP']):
        os.mkdir(os.environ['GLITE_LOCATION_TMP'],0755)

    if not params.has_key('X509_VOMS_DIR'):
        params['X509_VOMS_DIR'] = '/etc/grid-security/vomsdir'
    glib.export('X509_VOMS_DIR', params['X509_VOMS_DIR'])

    glib.export("JAVA_HOME","/usr/java/j2sdk1.4.2_04")
    glib.export("CATALINA_HOME","/var/lib/tomcat5")
    glib.export("GLOBUS_LOCATION")
    glib.export("GPT_LOCATION")

    glib.addEnvPath("PATH", "%s/bin" % os.environ["GLOBUS_LOCATION"])
    glib.addEnvPath("PATH", "%s/externals/bin" % os.environ["GLITE_LOCATION"])
    glib.addEnvPath("PATH", "%s/bin" % os.environ["GLITE_LOCATION"])
    glib.addEnvPath("PATH", "%s/bin" % os.environ["JAVA_HOME"])
    
    glib.addEnvPath("LD_LIBRARY_PATH", "%s/lib" % os.environ["GLOBUS_LOCATION"])
    glib.addEnvPath("LD_LIBRARY_PATH", "%s/lib" % os.environ["GLITE_LOCATION"])
    glib.addEnvPath("LD_LIBRARY_PATH", "%s/lib/" % params['catalog-service-fr.oracle-instantclient.location'])

    # Perl
    glib.addEnvPath("PERL5LIB", "%s/lib/perl:%s/lib/perl5" % (os.environ['GPT_LOCATION'],os.environ['GLITE_LOCATION']))

    # MANPATH
    glib.addEnvPath('MANPATH', os.environ['GLOBUS_LOCATION'] + '/man')
    glib.addEnvPath('MANPATH', os.environ['GLITE_LOCATION'] + '/share/man')

def replaceTags(line, params, prefix):
    if line.find("@") > -1:
        tags = line.split("@")
        i = 1
        while i < len(tags):
            if params.has_key(prefix + "." + tags[i]):
                paramName = "%s.%s" % (prefix, tags[i])
                if verbose:
                    print("Replace @%s@ by %s" % (tags[i], params[paramName]))
                line = line.replace("@%s@" % tags[i], params[paramName])
            else:
                if verbose:
                    print("Replace @%s@ by empty string" %(tags[i]))
                line = line.replace("@%s@" % tags[i], "")
            i += 2
    return line

#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # The script must be run as root
    if not os.geteuid()==0:
        print '"\nThis script must be run as root\n'
        sys.exit(1)
        
    # Get an instance of the ConfigParams class
    params = ConfigParams()
    
    # Get an instance of the library class
    glib = gLib()
    
    # Load parameters
    loadDefaults(params)
    try:
        opts, args = glib.getopt(sys.argv[1:], '', ['siteconfig='])
        for o, a in opts:
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
    except getopt.GetoptError:
        pass
    if glib.loadConfiguration(["%s/../glite-data-single-catalog-oracle.cfg.xml" % glib.getScriptPath(), "%s/../glite-rgma-servicetool.cfg.xml" % glib.getScriptPath()],params):
        print "An error occurred while configuring the service"
        sys.exit(1)

    params['gLib'] = glib
    
    verbose = 0
    if params.has_key('glite.installer.verbose'):
        if params['glite.installer.verbose'] == "true":
            verbose = 1
    glib.verbose = verbose
    
    # Set up the environment
    set_env()
          
    #Instantiate grdSecurity classes
    gsec = gridSecurity(glib)
    gsec.setVerbosity(verbose)
    gsec.params = params

    # Instantiate the service classes
    service = glite_data_single_catalog_oracle()
    service.verbose = verbose

    # Instantiate the rgma servicetool class
    rgmaServicetool = gliteRgmaServicetool()
    rgmaServicetool.verbose = verbose

    # Command line opts if any
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version', 'start', 'stop', 'remove', 'status', 'configure', 'siteconfig='])
    except getopt.GetoptError:
        service.usage(msg = "Unknown options(s)")
        sys.exit(1)

    # ------------------------------------------------------------
    # Check cli options
    # ------------------------------------------------------------    
    for o, a in opts:
        
        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)
            
        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)
            
        if o in ("-c", "--checkconf"):
            service.copyright()
            service.showVersion()
            glib.print_params(params)
            sys.exit(0)
            
        if o == "--configure":
            # Check certificates
            if params.has_key('glite.installer.checkcerts'):
                if params['glite.installer.checkcerts'] == "true":
                    if glib.check_certs(params) != 0:
                        print "An error occurred while configuring the %s" \
                            % service.friendly_name
                        sys.exit(1)
            
            service.copyright()
            service.showVersion()
            service.banner()
                
            # Display the parameters if verbose output
            if verbose:
                glib.print_params(params)
                
            # Stop all services
            glib.printInfoMessage("Stopping all running Fireman services...")
            service.stop()
            
            # Configure the service
            return_value = service.configure()
            
            if return_value == 0:

                # Stop all services
                glib.printInfoMessage("Stopping all running Fireman services...")
                service.stop()
                
                glib.registerService()

                print "\n\nThe %s configuration was successfully completed\n" % service.friendly_name
                print "\nBefore using the %s services make sure that" % service.friendly_name
                print "the host certificate of each used VOMS server is installed in"
                print "%s\n" % params['X509_VOMS_DIR']
                print "You can now start the service using the --start option of this script\n\n"
    
                sys.exit(0)

            elif return_value == 2:

                # Stop all services
                glib.printInfoMessage("Stopping all running Fireman services...")
                service.stop()
                
                print "\n\nThe %s configuration was completed, but one or more warnings" % service.friendly_name
                print "were issued. Please revise them and re-run the script"
                print "or configure and start the %s manually\n" % service.friendly_name

                sys.exit(2)

            else:
                print "\n\nAn unrecoverable error occurred while configuring the %s" \
                    % service.friendly_name

                sys.exit(1)
            
        if o == "--start":
            if service.start() == 0:
                print "\n\nThe %s were successfully started\n" % service.friendly_name
                sys.exit(0)
            else:
                print "\n\nAn unrecoverable error occurred while starting the %s" \
                    % service.friendly_name
                sys.exit(1)

        if o == "--stop":
            if service.stop() == 0:
                print "\n\nThe %s were successfully stopped\n" % service.friendly_name
                sys.exit(0)
            else:
                print "\n\nAn unrecoverable error occurred while stopping the %s" \
                    % service.friendly_name
                sys.exit(1)

        if o == "--status":
            errorcode = service.status()
            if errorcode:
                sys.exit(1)
            sys.exit(0)

        if o == "--remove":
            glib.printInfoMessage("\n\nStopping all running services to remove the DB schema...")
            if service.stop() != 0:
                glib.printErrorMessage("Could not stop all services. Please verify and try again")
                sys.exit(1)
            sys.exit(service.remove())
